@@ -204,7 +204,20 @@ let term : unit Term.t =
     @@ fun () ->
     let open Fiber.O in
     let* setup = Import.Main.setup () in
-    build_exn @@ step ~setup ~prog ~args ~common ~no_rebuild ~context ~on_exit:exit
+    build_exn (fun () ->
+      let open Memo.O in
+      let* sctx = setup >>| Import.Main.find_scontext_exn ~name:context in
+      let* env = Super_context.context_env sctx in
+      let expand = Cmd_arg.expand ~root:(Common.root common) ~sctx in
+      let* prog =
+        let dir =
+          let context = Dune_rules.Super_context.context sctx in
+          Path.Build.relative (Context.build_dir context) (Common.prefix_target common "")
+        in
+        let* prog = expand prog in
+        get_path_and_build_if_necessary sctx ~no_rebuild ~dir ~prog >>| Path.to_string
+      and* args = Memo.parallel_map ~f:expand args in
+      restore_cwd_and_execve (Common.root common) prog args env)
 ;;
 
 let command = Cmd.v info term
