@@ -1,4 +1,5 @@
 open Import
+module Diff_promotion = Promote.Diff_promotion
 
 let files_to_promote ~common files : Diff_promotion.files_to_promote =
   match files with
@@ -14,6 +15,21 @@ let files_to_promote ~common files : Diff_promotion.files_to_promote =
     These (files, on_missing)
 ;;
 
+let display_files files_to_promote =
+  let open Fiber.O in
+  Diff_promotion.load_db ()
+  |> Diff_promotion.filter_db files_to_promote
+  |> Fiber.parallel_map ~f:(fun file ->
+    Diff_promotion.diff_for_file file
+    >>| function
+    | Ok _ -> Some file
+    | Error _ -> None)
+  >>| List.filter_opt
+  >>| List.sort ~compare:(fun file file' -> Diff_promotion.File.compare file file')
+  >>| List.iter ~f:(fun (file : Diff_promotion.File.t) ->
+    Console.printf "%s" (Diff_promotion.File.source file |> Path.Source.to_string))
+;;
+
 module Apply = struct
   let info =
     let doc = "Promote files from the last run" in
@@ -53,7 +69,23 @@ module Diff = struct
     and+ files = Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE") in
     let common, config = Common.init builder in
     let files_to_promote = files_to_promote ~common files in
-    Scheduler.go ~common ~config (fun () -> Diff_promotion.display files_to_promote)
+    Scheduler.go_with_rpc_server ~common ~config (fun () ->
+      Diff_promotion.display files_to_promote)
+  ;;
+
+  let command = Cmd.v info term
+end
+
+module Files = struct
+  let info = Cmd.info ~doc:"List promotions files" "list"
+
+  let term =
+    let+ builder = Common.Builder.term
+    and+ files = Arg.(value & pos_all Cmdliner.Arg.file [] & info [] ~docv:"FILE") in
+    let common, config = Common.init builder in
+    let files_to_promote = files_to_promote ~common files in
+    Scheduler.go_with_rpc_server ~common ~config (fun () ->
+      display_files files_to_promote)
   ;;
 
   let command = Cmd.v info term
@@ -63,7 +95,7 @@ let info =
   Cmd.info ~doc:"Control how changes are propagated back to source code." "promotion"
 ;;
 
-let group = Cmd.group info [ Apply.command; Diff.command ]
+let group = Cmd.group info [ Files.command; Apply.command; Diff.command ]
 
 let promote =
   command_alias ~orig_name:"promotion apply" Apply.command Apply.term "promote"
