@@ -26,8 +26,8 @@ let info =
 ;;
 
 let term =
-  let+ (common : Common.t) = Common.term in
-  Rpc_common.client_term common exec
+  let+ (builder : Common.Builder.t) = Common.Builder.term in
+  Rpc_common.client_term builder exec
 ;;
 
 let cmd = Cmd.v info term
