@@ -488,6 +488,7 @@ module External : sig
   include Path_intf.S
   module Table : Hashtbl.S with type key = t
 
+  val root : t
   val relative : t -> string -> t
   val mkdir_p : ?perms:int -> t -> unit
   val initial_cwd : t
@@ -495,22 +496,12 @@ module External : sig
   val as_local : t -> string
   val append_local : t -> Local.t -> t
   val of_filename_relative_to_initial_cwd : string -> t
-  val drop_prefix : t -> prefix:t -> Local.t option
+  val is_broken_symlink : t -> bool
 end = struct
   module Table = String.Table
 
   type t = string
 
-  let drop_prefix path ~prefix =
-    if prefix = path
-    then Some Local.root
-    else (
-      let prefix = prefix ^ "/" in
-      let open Option.O in
-      let+ suffix = String.drop_prefix path ~prefix in
-      Local.of_string suffix)
-  ;;
-
   let to_string t = t
   let equal = String.equal
   let hash = String.hash
@@ -581,6 +572,8 @@ end = struct
     if Filename.is_relative fn then relative initial_cwd fn else of_string fn
   ;;
 
+  let is_broken_symlink = Fpath.is_broken_symlink
+
   include (
     Comparator.Operators (struct
       type nonrec t = t
@@ -1074,6 +1067,12 @@ let as_in_source_tree_exn t =
       [ "t", to_dyn t ]
 ;;
 
+let as_outside_build_dir : t -> Outside_build_dir.t option = function
+  | In_source_tree s -> Some (In_source_dir s)
+  | External s -> Some (External s)
+  | In_build_dir _ -> None
+;;
+
 let as_outside_build_dir_exn : t -> Outside_build_dir.t = function
   | In_source_tree s -> In_source_dir s
   | External s -> External s
@@ -1454,13 +1453,23 @@ let chmod t ~mode = Unix.chmod (to_string t) mode
 let follow_symlink path = Fpath.follow_symlink (to_string path) |> Result.map ~f:of_string
 
 let drop_prefix path ~prefix =
-  if prefix = path
-  then Some Local.root
-  else (
-    let prefix = to_string prefix ^ "/" in
+  let prefix_s = to_string prefix in
+  let path_s = to_string path in
+  match Int.compare (String.length prefix_s) (String.length path_s) with
+  | Eq -> if prefix = path then Some Local.root else None
+  | Gt -> None
+  | Lt ->
     let open Option.O in
-    let+ suffix = String.drop_prefix (to_string path) ~prefix in
-    Local.of_string suffix)
+    let* prefix =
+      let* last = String.last prefix_s in
+      if is_dir_sep last
+      then Some prefix_s
+      else if is_dir_sep path_s.[String.length prefix_s]
+      then Some (prefix_s ^ String.make 1 path_s.[String.length prefix_s])
+      else None
+    in
+    let+ suffix = String.drop_prefix path_s ~prefix in
+    Local.of_string suffix
 ;;
 
 let drop_prefix_exn t ~prefix =
@@ -1470,6 +1479,14 @@ let drop_prefix_exn t ~prefix =
   | Some p -> p
 ;;
 
+let is_broken_symlink = function
+  | External e -> External.is_broken_symlink e
+  | In_source_tree _ | In_build_dir _ ->
+    (* Paths within the source tree and build dir are always fully-expanded,
+       so there's no possibility for them to be broken symlinks. *)
+    false
+;;
+
 module Expert = struct
   let drop_absolute_prefix ~prefix p =
     match
