@@ -1,51 +1,4 @@
-open Stdune
-open Dune_sexp
-
-module Op = struct
-  type t =
-    | Eq
-    | Gt
-    | Gte
-    | Lte
-    | Lt
-    | Neq
-
-  let eval t (x : Ordering.t) =
-    match t, x with
-    | (Eq | Gte | Lte), Eq | (Neq | Lt | Lte), Lt | (Neq | Gt | Gte), Gt -> true
-    | _, _ -> false
-  ;;
-
-  let to_dyn =
-    let open Dyn in
-    function
-    | Eq -> string "Eq"
-    | Gt -> string "Gt"
-    | Gte -> string "Gte"
-    | Lte -> string "Lte"
-    | Lt -> string "Lt"
-    | Neq -> string "Neq"
-  ;;
-
-  let equal a b =
-    match a, b with
-    | Eq, Eq -> true
-    | Gt, Gt -> true
-    | Gte, Gte -> true
-    | Lte, Lte -> true
-    | Lt, Lt -> true
-    | Neq, Neq -> true
-    | _ -> false
-  ;;
-
-  let by_string = [ "=", Eq; ">=", Gte; "<=", Lte; ">", Gt; "<", Lt; "<>", Neq ]
-  let to_string t = fst (List.find_exn by_string ~f:(fun (_, op) -> equal op t))
-
-  let encode t =
-    let open Encoder in
-    string (to_string t)
-  ;;
-end
+open Import
 
 type 'string ast =
   | Const of bool
@@ -53,7 +6,7 @@ type 'string ast =
   | Expr of 'string
   | And of 'string ast list
   | Or of 'string ast list
-  | Compare of Op.t * 'string * 'string
+  | Compare of Relop.t * 'string * 'string
 
 module Ast = struct
   type 'string t = 'string ast
@@ -61,6 +14,17 @@ module Ast = struct
   let true_ = Const true
   let false_ = Const false
 
+  let rec equal f t1 t2 =
+    match t1, t2 with
+    | Const b1, Const b2 -> Bool.equal b1 b2
+    | Not t1, Not t2 -> equal f t1 t2
+    | Expr x1, Expr x2 -> f x1 x2
+    | And tl1, And tl2 | Or tl1, Or tl2 -> List.equal (equal f) tl1 tl2
+    | Compare (op1, x1, y1), Compare (op2, x2, y2) ->
+      Relop.equal op1 op2 && f x1 x2 && f y1 y2
+    | (Const _ | Not _ | Expr _ | And _ | Or _ | Compare _), _ -> false
+  ;;
+
   let rec to_dyn string_to_dyn =
     let open Dyn in
     function
@@ -70,13 +34,18 @@ module Ast = struct
     | And t -> variant "And" (List.map ~f:(to_dyn string_to_dyn) t)
     | Or t -> variant "Or" (List.map ~f:(to_dyn string_to_dyn) t)
     | Compare (o, s1, s2) ->
-      variant "Compare" [ Op.to_dyn o; string_to_dyn s1; string_to_dyn s2 ]
+      variant "Compare" [ Relop.to_dyn o; string_to_dyn s1; string_to_dyn s2 ]
   ;;
 
-  let decode decode_string =
+  let decode ~override_decode_bare_literal decode_string =
     let open Decoder in
+    let decode_bare_literal =
+      match override_decode_bare_literal with
+      | None -> decode_string
+      | Some decode_bare_literal -> decode_bare_literal
+    in
     let ops =
-      List.map Op.by_string ~f:(fun (name, op) ->
+      List.map Relop.map ~f:(fun (name, op) ->
         ( name
         , let+ x = decode_string
           and+ y = decode_string in
@@ -84,13 +53,16 @@ module Ast = struct
     in
     let decode =
       fix (fun t ->
-        sum
-          ~force_parens:true
-          (("or", repeat t >>| fun x -> Or x)
-           :: ("and", repeat t >>| fun x -> And x)
-           :: ("not", Syntax.since Stanza.syntax (3, 2) >>> t >>| fun x -> Not x)
-           :: ops)
-        <|> let+ v = decode_string in
+        let decode_term =
+          sum
+            ~force_parens:true
+            (("or", repeat t >>| fun x -> Or x)
+             :: ("and", repeat t >>| fun x -> And x)
+             :: ("not", Syntax.since Stanza.syntax (3, 2) >>> t >>| fun x -> Not x)
+             :: ops)
+        in
+        decode_term
+        <|> let+ v = decode_bare_literal in
             Expr v)
     in
     let+ () = Syntax.since Stanza.syntax (1, 1)
@@ -107,7 +79,7 @@ module Ast = struct
     | Expr e -> encode_string e
     | And ts -> List (string "and" :: List.map ts ~f:(encode encode_string))
     | Or ts -> List (string "or" :: List.map ts ~f:(encode encode_string))
-    | Compare (o, s1, s2) -> List [ Op.encode o; encode_string s1; encode_string s2 ]
+    | Compare (o, s1, s2) -> List [ Relop.encode o; encode_string s1; encode_string s2 ]
   ;;
 end
 
@@ -116,5 +88,6 @@ type t = String_with_vars.t ast
 let true_ = Ast.true_
 let false_ = Ast.false_
 let to_dyn = Ast.to_dyn String_with_vars.to_dyn
-let decode = Ast.decode String_with_vars.decode
+let decode = Ast.decode ~override_decode_bare_literal:None String_with_vars.decode
 let encode = Ast.encode String_with_vars.encode
+let equal = Ast.equal String_with_vars.equal
