@@ -1,5 +1,4 @@
-open! Stdune
-open Dune_sexp
+open Import
 open Pp.O
 module Format = Stdlib.Format
 
@@ -27,7 +26,7 @@ let print_wrapped_list ~version x =
   if version < (2, 8)
   then Pp.char '(' ++ Pp.hovbox ~indent:1 inner ++ Pp.char ')'
   else
-    (if version < (3, 19) then Pp.hvbox else Pp.hovbox)
+    (if version < (3, 20) then Pp.hvbox else Pp.hovbox)
       ~indent:1
       (Pp.char '(' ++ inner ++ Pp.char ')')
 ;;
