@@ -1,5 +1,4 @@
-open Stdune
-open Dune_sexp
+open Import
 module Payload = Template.Pform.Payload
 
 module Var : sig
@@ -32,6 +31,7 @@ module Var : sig
       | Group
       | Jobs
       | Arch
+      | Sys_ocaml_version
       | Section_dir of Section.t
 
     val compare : t -> t -> Ordering.t
@@ -117,6 +117,7 @@ module Macro : sig
         }
     | Lib_available
     | Bin_available
+    | File_available
     | Version
     | Read
     | Read_strings
