@@ -1,4 +1,4 @@
-open! Stdune
+open Import
 
 type t =
   | String of string
