@@ -7,11 +7,13 @@ module Pkg_info : sig
     { name : Package_name.t
     ; version : Package_version.t
     ; dev : bool
+    ; avoid : bool
     ; source : Source.t option
     ; extra_sources : (Path.Local.t * Source.t) list
     }
 
   val default_version : Package_version.t
+  val variables : t -> OpamVariable.variable_contents Package_variable_name.Map.t
 end
 
 module Build_command : sig
@@ -25,12 +27,19 @@ module Pkg : sig
     { build_command : Build_command.t option
     ; install_command : Action.t option
     ; depends : (Loc.t * Package_name.t) list
+    ; depexts : string list
     ; info : Pkg_info.t
     ; exported_env : String_with_vars.t Action.Env_update.t list
     }
 
+  val remove_locs : t -> t
   val equal : t -> t -> bool
   val decode : (lock_dir:Path.Source.t -> Package_name.t -> t) Decoder.t
+  val files_dir : Package_name.t -> lock_dir:Path.Source.t -> Path.Source.t
+end
+
+module Package_filename : sig
+  val of_package_name : Package_name.t -> string
 end
 
 module Repositories : sig
@@ -41,13 +50,13 @@ type t = private
   { version : Syntax.Version.t
   ; dependency_hash : (Loc.t * Local_package.Dependency_hash.t) option
   ; packages : Pkg.t Package_name.Map.t
-  (** It's guaranteed that this map will contain an entry for all dependencies
+    (** It's guaranteed that this map will contain an entry for all dependencies
       of all packages in this map. That is, the set of packages is closed under
       the "depends on" relationship between packages. *)
   ; ocaml : (Loc.t * Package_name.t) option
   ; repos : Repositories.t
   ; expanded_solver_variable_bindings : Solver_stats.Expanded_variable_bindings.t
-  (** Stores the solver variables that were evaluated while solving
+    (** Stores the solver variables that were evaluated while solving
       dependencies. Can be used to determine if a lockdir is compatible
       with a particular system. *)
   }
@@ -71,6 +80,10 @@ val create_latest_version
 
 val default_path : Path.Source.t
 
+(** Returns the path to the lockdir that will be used to lock the
+    given dev tool *)
+val dev_tool_lock_dir_path : Dev_tool.t -> Path.Source.t
+
 module Metadata : Dune_sexp.Versioned_file.S with type data := unit
 
 val metadata_filename : Filename.t
@@ -88,7 +101,8 @@ module Write_disk : sig
   val commit : t -> unit
 end
 
-val read_disk : Path.Source.t -> t
+val read_disk : Path.Source.t -> (t, User_message.t) result
+val read_disk_exn : Path.Source.t -> t
 
 module Make_load (Io : sig
     include Monad.S
@@ -98,7 +112,8 @@ module Make_load (Io : sig
     val with_lexbuf_from_file : Path.Source.t -> f:(Lexing.lexbuf -> 'a) -> 'a t
     val stats_kind : Path.Source.t -> (File_kind.t, Unix_error.Detailed.t) result t
   end) : sig
-  val load : Path.Source.t -> t Io.t
+  val load : Path.Source.t -> (t, User_message.t) result Io.t
+  val load_exn : Path.Source.t -> t Io.t
 end
 
 (** [transitive_dependency_closure t names] returns the set of package names
