@@ -21,42 +21,43 @@ let lockdir_regenerate_hints =
 let version_by_package_name local_packages (lock_dir : Lock_dir.t) =
   let from_local_packages =
     Package_name.Map.map local_packages ~f:(fun (local_package : Local_package.t) ->
-      Option.value local_package.version ~default:Lock_dir.Pkg_info.default_version)
+      local_package.version)
   in
   let from_lock_dir =
     Package_name.Map.map lock_dir.packages ~f:(fun pkg -> pkg.info.version)
   in
   let exception Duplicate_package of Package_name.t in
   try
-    Ok
-      (Package_name.Map.union
-         from_local_packages
-         from_lock_dir
-         ~f:(fun duplicate_package_name _ _ ->
-           raise (Duplicate_package duplicate_package_name)))
+    Package_name.Map.union
+      from_local_packages
+      from_lock_dir
+      ~f:(fun duplicate_package_name _ _ ->
+        raise (Duplicate_package duplicate_package_name))
   with
   | Duplicate_package duplicate_package_name ->
     let local_package = Package_name.Map.find_exn local_packages duplicate_package_name in
-    Error
-      (User_message.make
-         ~hints:lockdir_regenerate_hints
-         ~loc:local_package.loc
-         [ Pp.textf
-             "A package named %S is defined locally but is also present in the lockdir"
-             (Package_name.to_string local_package.name)
-         ])
+    User_error.raise
+      ~hints:lockdir_regenerate_hints
+      ~loc:local_package.loc
+      [ Pp.textf
+          "A package named %S is defined locally but is also present in the lockdir"
+          (Package_name.to_string local_package.name)
+      ]
 ;;
 
 let concrete_dependencies_of_local_package t local_package_name ~with_test =
   let local_package = Package_name.Map.find_exn t.local_packages local_package_name in
-  Local_package.(for_solver local_package |> For_solver.opam_filtered_dependency_formula)
-  |> Resolve_opam_formula.filtered_formula_to_package_names
-       ~with_test
-       (Solver_env.to_env t.solver_env)
-       t.version_by_package_name
-  |> Result.map_error ~f:(function
-    | `Formula_could_not_be_satisfied unsatisfied_formula_hints ->
-    User_message.make
+  match
+    (Local_package.for_solver local_package).dependencies
+    |> Dependency_formula.to_filtered_formula
+    |> Resolve_opam_formula.filtered_formula_to_package_names
+         ~with_test
+         ~env:(Solver_env.to_env t.solver_env)
+         ~packages:t.version_by_package_name
+  with
+  | Ok { regular; post = _ } -> regular
+  | Error (`Formula_could_not_be_satisfied unsatisfied_formula_hints) ->
+    User_error.raise
       ?hints:(Option.some_if with_test lockdir_regenerate_hints)
       ~loc:local_package.loc
       (Pp.textf
@@ -67,17 +68,16 @@ let concrete_dependencies_of_local_package t local_package_name ~with_test =
           else " when the solver variable 'with_test' is set to 'false'")
        :: List.map
             unsatisfied_formula_hints
-            ~f:Resolve_opam_formula.Unsatisfied_formula_hint.pp))
+            ~f:Resolve_opam_formula.Unsatisfied_formula_hint.pp)
 ;;
 
 let all_non_local_dependencies_of_local_packages t =
-  let open Result.O in
-  let+ all_dependencies_of_local_packages =
+  let all_dependencies_of_local_packages =
     Package_name.Map.keys t.local_packages
-    |> Result.List.map ~f:(fun p ->
+    |> List.map ~f:(fun p ->
       concrete_dependencies_of_local_package ~with_test:true t p
-      |> Result.map ~f:Package_name.Set.of_list)
-    |> Result.map ~f:Package_name.Set.union_all
+      |> Package_name.Set.of_list)
+    |> Package_name.Set.union_all
   in
   Package_name.Set.diff
     all_dependencies_of_local_packages
@@ -85,14 +85,14 @@ let all_non_local_dependencies_of_local_packages t =
 ;;
 
 let check_for_unnecessary_packges_in_lock_dir
-  t
-  all_non_local_dependencies_of_local_packages
+      lock_dir
+      all_non_local_dependencies_of_local_packages
   =
   let unneeded_packages_in_lock_dir =
     let locked_transitive_closure_of_local_package_dependencies =
       match
         Lock_dir.transitive_dependency_closure
-          t.lock_dir
+          lock_dir
           all_non_local_dependencies_of_local_packages
       with
       | Ok x -> x
@@ -102,33 +102,52 @@ let check_for_unnecessary_packges_in_lock_dir
           "Missing packages from lockdir after confirming no missing packages in lockdir"
           [ "missing package", Package_name.Set.to_dyn missing_packages ]
     in
-    let all_locked_packages = Package_name.Set.of_keys t.lock_dir.packages in
+    let all_locked_packages = Package_name.Set.of_keys lock_dir.packages in
     Package_name.Set.diff
       all_locked_packages
       locked_transitive_closure_of_local_package_dependencies
   in
   if Package_name.Set.is_empty unneeded_packages_in_lock_dir
-  then Ok ()
+  then ()
   else (
     let packages =
       Package_name.Set.to_list unneeded_packages_in_lock_dir
-      |> List.map ~f:(Package_name.Map.find_exn t.lock_dir.packages)
+      |> List.map ~f:(Package_name.Map.find_exn lock_dir.packages)
     in
-    Error
-      (User_message.make
-         ~hints:lockdir_regenerate_hints
-         [ Pp.text
-             "The lockdir contains packages which are not among the transitive \
-              dependencies of any local package:"
-         ; Pp.enumerate packages ~f:(fun (package : Lock_dir.Pkg.t) ->
-             Pp.textf
-               "%s.%s"
-               (Package_name.to_string package.info.name)
-               (Package_version.to_string package.info.version))
-         ]))
+    User_error.raise
+      ~hints:lockdir_regenerate_hints
+      [ Pp.text
+          "The lockdir contains packages which are not among the transitive dependencies \
+           of any local package:"
+      ; Pp.enumerate packages ~f:(fun (package : Lock_dir.Pkg.t) ->
+          Pp.textf
+            "%s.%s"
+            (Package_name.to_string package.info.name)
+            (Package_version.to_string package.info.version))
+      ])
 ;;
 
-let validate_dependency_hash { local_packages; lock_dir; _ } =
+let up_to_date local_packages ~dependency_hash:saved_dependency_hash =
+  let local_packages =
+    Package_name.Map.values local_packages |> List.map ~f:Local_package.for_solver
+  in
+  let dependency_hash =
+    Local_package.For_solver.non_local_dependencies local_packages
+    |> Local_package.Dependency_hash.of_dependency_formula
+  in
+  match saved_dependency_hash, dependency_hash with
+  | None, None -> `Valid
+  | Some lock_dir_dependency_hash, Some non_local_dependencies_hash
+    when Local_package.Dependency_hash.equal
+           lock_dir_dependency_hash
+           non_local_dependencies_hash -> `Valid
+  | None, Some _ ->
+    `Valid (* This case happens when the user writes themselves their lock.dune. *)
+  | Some _, Some _ -> `Invalid
+  | Some _, None -> `Invalid
+;;
+
+let validate_dependency_hash local_packages ~saved_dependency_hash =
   let local_packages =
     Package_name.Map.values local_packages |> List.map ~f:Local_package.for_solver
   in
@@ -140,73 +159,82 @@ let validate_dependency_hash { local_packages; lock_dir; _ } =
         ]
     ]
   in
-  let non_local_dependencies =
-    Local_package.For_solver.list_non_local_dependency_set local_packages
+  let dependency_hash =
+    Local_package.For_solver.non_local_dependencies local_packages
+    |> Local_package.Dependency_hash.of_dependency_formula
   in
-  let dependency_hash = Local_package.Dependency_set.hash non_local_dependencies in
-  match lock_dir.dependency_hash, dependency_hash with
-  | None, None -> Ok ()
+  match saved_dependency_hash, dependency_hash with
+  | None, None -> ()
   | Some (loc, lock_dir_dependency_hash), None ->
-    Error
-      (User_error.make
-         ~loc
-         ~hints:regenerate_lock_dir_hints
-         [ Pp.textf
-             "This project has no non-local dependencies yet the lockfile contains a \
-              dependency hash: %s"
-             (Local_package.Dependency_hash.to_string lock_dir_dependency_hash)
-         ])
+    User_error.raise
+      ~loc
+      ~hints:regenerate_lock_dir_hints
+      [ Pp.textf
+          "This project has no non-local dependencies yet the lockfile contains a \
+           dependency hash: %s"
+          (Local_package.Dependency_hash.to_string lock_dir_dependency_hash)
+      ]
   | None, Some _ ->
-    let any_non_local_dependency : Package_dependency.t =
-      List.hd (Local_package.Dependency_set.package_dependencies non_local_dependencies)
+    let any_non_local_dependency_name =
+      let non_local_dependencies =
+        Local_package.For_solver.non_local_dependencies local_packages
+      in
+      match Dependency_formula.any_package_name non_local_dependencies with
+      | Some x -> x
+      | None ->
+        Code_error.raise
+          "Attempting to retrieve a non-local dependency but there aren't any"
+          []
     in
-    Error
-      (User_error.make
-         ~hints:regenerate_lock_dir_hints
-         [ Pp.text
-             "This project has at least one non-local dependency but the lockdir doesn't \
-              contain a dependency hash."
-         ; Pp.textf
-             "An example of a non-local dependency of this project is: %s"
-             (Package_name.to_string any_non_local_dependency.name)
-         ])
+    User_error.raise
+      ~hints:regenerate_lock_dir_hints
+      [ Pp.text
+          "This project has at least one non-local dependency but the lockdir doesn't \
+           contain a dependency hash."
+      ; Pp.textf
+          "An example of a non-local dependency of this project is: %s"
+          (Package_name.to_string any_non_local_dependency_name)
+      ]
   | Some (loc, lock_dir_dependency_hash), Some non_local_dependency_hash ->
-    if Local_package.Dependency_hash.equal
-         lock_dir_dependency_hash
-         non_local_dependency_hash
-    then Ok ()
+    if
+      Local_package.Dependency_hash.equal
+        lock_dir_dependency_hash
+        non_local_dependency_hash
+    then ()
     else
-      Error
-        (User_error.make
-           ~loc
-           ~hints:regenerate_lock_dir_hints
-           [ Pp.text
-               "Dependency hash in lockdir does not match the hash of non-local \
-                dependencies of this project. The lockdir expects the the non-local \
-                dependencies to hash to:"
-           ; Pp.text (Local_package.Dependency_hash.to_string lock_dir_dependency_hash)
-           ; Pp.text "...but the non-local dependencies of this project hash to:"
-           ; Pp.text (Local_package.Dependency_hash.to_string non_local_dependency_hash)
-           ])
+      User_error.raise
+        ~loc
+        ~hints:regenerate_lock_dir_hints
+        [ Pp.text
+            "Dependency hash in lockdir does not match the hash of non-local \
+             dependencies of this project. The lockdir expects the the non-local \
+             dependencies to hash to:"
+        ; Pp.text (Local_package.Dependency_hash.to_string lock_dir_dependency_hash)
+        ; Pp.text "...but the non-local dependencies of this project hash to:"
+        ; Pp.text (Local_package.Dependency_hash.to_string non_local_dependency_hash)
+        ]
 ;;
 
 let validate t =
-  let open Result.O in
-  let* () = validate_dependency_hash t in
+  validate_dependency_hash
+    t.local_packages
+    ~saved_dependency_hash:t.lock_dir.dependency_hash;
   all_non_local_dependencies_of_local_packages t
-  >>= check_for_unnecessary_packges_in_lock_dir t
+  |> check_for_unnecessary_packges_in_lock_dir t.lock_dir
 ;;
 
 let create local_packages lock_dir =
-  let open Result.O in
-  let* version_by_package_name = version_by_package_name local_packages lock_dir in
-  let solver_env =
-    Solver_stats.Expanded_variable_bindings.to_solver_env
-      lock_dir.expanded_solver_variable_bindings
-  in
-  let t = { local_packages; lock_dir; version_by_package_name; solver_env } in
-  let+ () = validate t in
-  t
+  try
+    let version_by_package_name = version_by_package_name local_packages lock_dir in
+    let solver_env =
+      Solver_stats.Expanded_variable_bindings.to_solver_env
+        lock_dir.expanded_solver_variable_bindings
+    in
+    let t = { local_packages; lock_dir; version_by_package_name; solver_env } in
+    validate t;
+    Ok t
+  with
+  | User_error.E e -> Error e
 ;;
 
 let local_transitive_dependency_closure_without_test =
@@ -216,7 +244,6 @@ let local_transitive_dependency_closure_without_test =
       Top_closure.top_closure
         ~deps:(fun a ->
           concrete_dependencies_of_local_package t a ~with_test:false
-          |> User_error.ok_exn
           |> List.filter ~f:(Package_name.Map.mem t.local_packages))
         ~key:Fun.id
         start
@@ -239,18 +266,18 @@ let transitive_dependency_closure_without_test t start =
       |> Package_name.Set.union_map ~f:(fun name ->
         let all_deps =
           concrete_dependencies_of_local_package t name ~with_test:false
-          |> User_error.ok_exn
           |> Package_name.Set.of_list
         in
         Package_name.Set.diff all_deps local_package_names)
     in
-    Lock_dir.transitive_dependency_closure
-      t.lock_dir
-      Package_name.Set.(
-        union
-          non_local_immediate_dependencies_of_local_transitive_dependency_closure
-          (diff start local_package_names))
-    |> function
+    match
+      Lock_dir.transitive_dependency_closure
+        t.lock_dir
+        Package_name.Set.(
+          union
+            non_local_immediate_dependencies_of_local_transitive_dependency_closure
+            (diff start local_package_names))
+    with
     | Ok x -> x
     | Error (`Missing_packages missing_packages) ->
       Code_error.raise
@@ -284,8 +311,8 @@ let all_dependencies t package ~traverse =
   check_contains_package t package;
   let immediate_deps =
     match concrete_dependencies_of_local_package t package ~with_test:true with
-    | Ok x -> Package_name.Set.of_list x
-    | Error e ->
+    | x -> Package_name.Set.of_list x
+    | exception User_error.E e ->
       Code_error.raise
         "Invalid package universe which should have already been validated"
         [ "error", Dyn.string (User_message.to_string e) ]
@@ -302,7 +329,6 @@ let non_test_dependencies t package ~traverse =
   match traverse with
   | `Immediate ->
     concrete_dependencies_of_local_package t package ~with_test:false
-    |> User_error.ok_exn
     |> Package_name.Set.of_list
   | `Transitive ->
     let closure =
