@@ -19,7 +19,6 @@ module Emit = struct
     ; compile_flags : Ordered_set_lang.Unexpanded.t
     ; allow_overlapping_dependencies : bool
     ; enabled_if : Blang.t
-    ; dune_version : Dune_lang.Syntax.Version.t
     }
 
   include Stanza.Make (struct
@@ -120,7 +119,7 @@ module Emit = struct
          let open Enabled_if in
          let allowed_vars = Any in
          decode ~allowed_vars ~since:None ()
-       and+ dune_version = Dune_lang.Syntax.get_exn Stanza.syntax in
+       in
        let preprocess =
          let init =
            let f libname = Preprocess.With_instrumentation.Ordinary libname in
@@ -144,7 +143,6 @@ module Emit = struct
        ; compile_flags
        ; allow_overlapping_dependencies
        ; enabled_if
-       ; dune_version
        })
   ;;
 
