@@ -33,9 +33,10 @@ let serve_once ~filename =
   let thread =
     Thread.create
       (fun server ->
-        Http.Server.accept server ~f:(fun write_end ->
-          Http.Server.respond_file write_end ~file:filename);
-        Http.Server.stop server)
+         Http.Server.accept server ~f:(fun session ->
+           let () = Http.Server.accept_request session in
+           Http.Server.respond_file session ~file:filename);
+         Http.Server.stop server)
       server
   in
   port, thread
@@ -144,7 +145,8 @@ let%expect_test "downloading, without any checksum" =
   run (download ~unpack:false ~port ~filename:"" ~target:(subdir destination));
   Thread.join server;
   print_endline "Finished successfully, no checksum verification";
-  [%expect {|
+  [%expect
+    {|
     Done downloading
     Finished successfully, no checksum verification |}]
 ;;
